% Constructor of the map from data to colors.
%
% function retobj = data_colormap (varargin)
%
% See discussion of constructors in <a href="matlab:helpwin 'FAESOR/classes/Contents'">classes/Contents</a>. 
% Parameters:
%    range =array with range_min, and range_max, 
%    colormap =color map, for instance jet, hsv, cadcolors, and so on
% 
% Example:
% dcm=data_colormap(struct('range',[min(Uz),max(Uz)],'colormap',jet(16)));
%

function retobj = data_colormap (varargin)
    class_name='data_colormap';
    parent=classbase;
    if (nargin == 0)
        self.rmin = [];
        self.rmax = [];
        self.colormap = [];
        self.ncolors = 0;
        self = class(self, class_name, parent);
        retobj = self;
        return;
    elseif (nargin == 1)
        arg = varargin{1};
        if strcmp(class(arg),class_name) % copy constructor.
            retobj = arg;
            return;
        else
            Parameters = varargin{1};
            self.rmin = min(Parameters.range);
            self.rmax = max(Parameters.range);
            self.colormap = hot;
            if (isfield(Parameters,'colormap'))
                self.colormap = Parameters.colormap;
            end
            self.ncolors = length(self.colormap(:,1));
            self = class(self, class_name, parent);
            retobj = self;
            return;
        end
    else
        error('Illegal arguments!');
        return;
    end
    return;
end
