% Compute element load vectors for nonzero essential boundary conditions
% corresponding to nonzero prescribed acoustic pressure acceleration.
% 
% function evs = nz_ebc_stiffness_loads(self, geom, P)
%
% Return an array of them so they may be assembled.
%    Arguments
%     geom=geometry field
%     ddP=acoustic pressure second-order time derivative field
%
function evs = nz_ebc_stiffness_loads(self, geom, ddP)
    gcells = get(self.feblock,'gcells');
    nfens = get(gcells,'nfens');
    dim = get(geom,'dim');
    % Integration rule
    integration_rule = get(self.feblock, 'integration_rule');
    pc = get(integration_rule, 'param_coords');
    w  = get(integration_rule, 'weights');
    npts_per_gcell = get(integration_rule, 'npts');
    % Precompute basis f. values + basis f. gradients wrt parametric coord
    for j=1:npts_per_gcell
        Ns{j} = bfun(gcells,pc(j,:));
        Nders{j} = bfundpar(gcells,pc(j,:));
    end
    % Material
    mat = get(self.feblock, 'mater');
    bulk = get(get(mat,'property'),'bulk');
    rho = get(get(mat,'property'),'rho');
    c2 = bulk/rho ;
    % Prepare some data: connectivity and the element matrices
    conns = get(gcells, 'conn'); % connectivity
    Fe = cell(size(conns,1),1);
    eqnums = cell(size(conns,1),1);
    xs =get(geom,'values');% retrieve the geometry information
    iF=0;
    for i=1:size(conns,1)
        conn =conns(i,:);
        pP = gather(ddP, conn, 'prescribed_values');
        if norm (pP) ~= 0
            iF=iF+1;
            eqnums{iF} =gather(ddP,conns(i,:),'eqnums');
            Fe{iF} =zeros(nfens,1);
            x=xs(conn,:);
            for j=1:npts_per_gcell
                J = Jacobian_matrix(gcells,Nders{j},x);
                Jac = Jacobian_volume(gcells,conn, Ns{j}, J, x);
                Ndersp = Nders{j}/J;
                Fe{iF} = Fe{iF} - Ns{j}*((Ns{j}'*pP) * (Jac/c2 * w(j)));
            end% Loop over quadrature points
        end
    end% Loop over elements
    evs  = elevecset(struct('vec',{Fe(1:iF)}, 'eqnums',{eqnums(1:iF)}));
end