% Compute element load vectors for to nonzero essential boundary conditions
%
% function evs = nz_ebc_loads(self, geom, u)
%
% Compute the element load vectors corresponding to nonzero essential boundary
% conditions (settlement of the supports).
% Return an array of them so they may be assembled.
%    Call as
% evs = nz_ebc_loads(feb, geom, u)
%     geom=geometry field
%     u=displacement field
%
function evs = nz_ebc_loads(self, geom, u)
    ems = stiffness (self, geom, u);
    gcells = get(self.feblock,'gcells');
    conns = get(gcells, 'conn'); % connectivity
    Fe = cell(size(conns,1),1);
    eqnums = cell(size(conns,1),1);
    mat=get(ems,'mat');
    % Now loop over all gcells in the block
    j=1;
    for i=1:count(gcells)
        conn = conns(i,:); % connectivity
        pu = gather(u, conn, 'prescribed_values');
        if norm (pu) ~= 0
            Ke = mat{i}; % element stiffness matrix
            Fe{j} =  -Ke*pu;
            eqnums{j} = gather(u, conn, 'eqnums');
            j=j+1;
        end
    end
    Fe=Fe(1:j-1);
    eqnums=eqnums(1:j-1);
    evs  = elevecset(struct('vec',{Fe}, 'eqnums',{eqnums}));
end
