% Compute the natural forces from the natural deformations.
%
% function PN = natural_forces (self, E, G, A, I2, I3, J, L, dN)
%
% Inputs:
% E, G= Young's and shear modulus, 
% A= cross-sectional area, 
% I2, I3=central moment of inertia of the cross-section about the x2 and x3 
% coordinate axis, 
% J=St Venant torsion constant, 
% L= current length of the element, 
% dN= column vector of natural deformations; see local_frames() 
% 
% Outputs:
%    PN = column vector of natural forces; 
%      PN(1)= axial force; 
%      PN(2)= symmetric bending moment in the plane x1-x2; 
%      PN(3)= anti-symmetric bending bending moment in the plane x1-x2;    
%      PN(4)= symmetric bending bending moment in the plane x1-x3;
%      PN(5)= anti-symmetric bending bending moment in the plane x1-x3; 
%      PN(6)= axial torque. 
% 
function PN = natural_forces (self, E, G, A, I2, I3, J, L, dN)
    DN= natural_stiffness(self, E, G, A, I2, I3, J, L);
    %     Natural forces
    PN= DN*dN;
    % Note that the non-constitutive stiffness due to pre-existing internal forces is currently omitted
end
