% Constructor of the finite element block class.
%    This class implements mechanics of small-strain deformation with arbitrarily large
%    deflections and rotations for two-node beam elements.
%
% Always takes either zero or one argument.  For no arguments, the default
% object is created.  For one argument, the argument may be either an
% object of the same type, in which case the copy is created; or, it may be
% a struct with the following fields representing options.
% Options: those recognized by feblock
%
function retobj = feblock_defor_tie_line (varargin)
    class_name='feblock_defor_tie_line';
    if (nargin == 0)
        parent = feblock;
        self.force_function= [];
        self.stiffness_function= [];
        self.direction=[0; 0; 1];
        retobj = class(self,class_name,parent);
        return;
    elseif (nargin == 1)
        arg = varargin{1};
        if strcmp(class(arg),class_name) % copy constructor.
            retobj = arg;
            return;
        else
            Parameters=varargin{1};
            parent = feblock(varargin{:});
            self.force_function=Parameters.force_function;
            self.stiffness_function=Parameters.stiffness_function;
            self.direction=Parameters.direction;
            self.direction=self.direction/norm(self.direction);
            retobj = class(self,class_name,parent);
            return;
        end
    else
        error('Illegal arguments!');
    end
end
