% Compute the strain energy stored in the block.
%    Call as
% se = strain_energy(feb, geom, u)
%     geom=geometry field
%     u=displacement field
%
function se = strain_energy(self, x, u)
gcells = get(self.feblock,'gcells');
ngcells = count(gcells);
nfens = get(gcells(1),'nfens');
dim = get(x,'dim');
% Pre-allocate the element matrices
ems(1:nfens*ngcells) = deal(elemat);
iem=1;
% Material
mat = get(self.feblock, 'mater');
E = get(mat, 'E');
nu = get(mat, 'nu');
G = E/2/(1+nu);
% global stabfact;
% stabfact=10^(-0); % conf
stabfact=0.100;
matstates = get(self.feblock, 'matstates');
% Make the inverse map from finite element nodes to geometric cells
f2gmap=fenode_to_gcell_map (struct ('gcells',gcells));
gmap=get (f2gmap,'gcell_map');
fenodemap=get (f2gmap,'fenodes');
% Now loop over all finite element nodes
se=0;
for nix=1:length(gmap)
    gl=gmap{nix};
    thisnn=fenodemap{nix};
    if ~isempty(gl) % This node has a element patch in this block
        % establish local numbering of all nodes of the patch @node nix
        lnmap =0*(1:length(gmap));
        for k=1:length(gl)
            kconn = get(gcells(gl(k)), 'conn'); % connectivity
            for j= 1:length(kconn)
                if (lnmap(kconn(j)) == 0) & (kconn(j) ~= thisnn)
                    lnmap(kconn(j))=kconn(j);
                end
            end
        end
        patchconn=lnmap(lnmap>0);
        % now store the local numbers
        for k=1:length(patchconn)
            lnmap(patchconn(k)) =k;
        end
        lnmap(thisnn) =length(patchconn)+1;
        c = gather(x, thisnn, 'values', 'noreshape'); % coordinates of central node
        % material tangent
        D = tangent_moduli (mat, struct ('ms',matstates{nix,1}));
        % preallocate the strain-displacement matrix
        Bnodal=zeros(3,dim*(length(patchconn)+1));
        Vpatch =0;
        for k=1:length(gl)
            kconn = get(gcells(gl(k)), 'conn'); % connectivity
            lx = gather(x, kconn, 'values', 'noreshape'); % coordinates of nodes
            lx(:,1)=lx(:,1)-c(1);
            lx(:,2)=lx(:,2)-c(2);
            Nder = Ndermat_param (gcells(gl(k)), [1 1]/3);
            [Nspatialder,detF] = Ndermat_spatial (gcells(gl(k)), Nder, lx);
            Ve = detF * thickness(gcells(gl(k)), [1 1]/3) * 1/2*1/3;% were taking one third of the triangle
            Vpatch=Vpatch+Ve;
            for j=1:3
                lk=lnmap(kconn(j));
                Be=[ Nspatialder(j,1) 0; 0 Nspatialder(j,2); Nspatialder(j,2) Nspatialder(j,1)];
                Bnodal(:,dim*(lk-1)+1:dim*lk)=Bnodal(:,dim*(lk-1)+1:dim*lk)+Ve*Be;
            end
        end
        Bnodal = Bnodal / Vpatch;
%         if nix == 1
%             Bnodal_v=Bnodal(1,:) +Bnodal(2,:)
%             disp (' in here')
%         end
        U=gather(u, [patchconn thisnn], 'values');
        se = se + 1/2*U'*Bnodal'*D*Bnodal*U * Vpatch;
    end
end
return;
