% Draw a graphic representation.
%
% function draw (self, gv, context)
%
%
% Input arguments
% self = self
% gv = graphic viewer
% context = struct
% with mandatory fields
%    x=reference geometry field
%    u=displacement field
% with optional fields
%    facecolor = color of the faces, solid
%    colorfield =field with vertex colors
%       only one of the facecolor and colorfield may be supplied
%    shrink = shrink factor
%
function draw (self, gv, context)
    conn = get(self, 'conn'); % connectivity
    x = gather(context.x, conn, 'values', 'noreshape'); % coordinates of nodes
    u = gather(context.u, conn, 'values', 'noreshape'); % coordinates of nodes
    % these are all defined in clockwise order, which is what Matlab expects
     ef=[1,9,2,18,6,13,5;
        2,10,3,19,7,14,6;
        3,11,4,20,8,15,7;
        4,12,1,17,5,16,8];
    f=[  1     9    21
     9     2    10
    21    10     3
    12    21    11
     5    16    26
    16     8    15
    26    15     7
    13    26    14
     1    17    22
    17     5    13
    22    13     6
     9    22    18
     2    18    23
    18     6    14
    23    14     7
    10    23    19
     3    19    24
    19     7    15
    24    15     8
    11    24    20
     4    20    25
    20     8    16
    25    16     5
    12    25    17
    21    12     1
    10    21     9
     3    11    21
    11     4    12
    26    13     5
    15    26    16
     7    14    26
    14     6    13
    22     9     1
    13    22    17
     6    18    22
    18     2     9
    23    10     2
    14    23    18
     7    19    23
    19     3    10
    24    11     3
    15    24    19
     8    20    24
    20     4    11
    25    12     4
    16    25    20
     5    17    25
    17     1    12];
    edgecolor = 'Black';
    if (isfield(context,'edgecolor'))
        edgecolor =context.edgecolor;
    end
    if (isfield(context,'colorfield'))
    else
        facecolor = 'red';
        if (isfield(context,'facecolor'))
            facecolor = context.facecolor;
        end
        edgecolor = 'Black';
        if (isfield(context,'edgecolor'))
            edgecolor = context.edgecolor;
        end
        options=struct ('facecolor',facecolor,'edgecolor',edgecolor);
    end
    conns = get(self, 'conn'); % connectivity
    shrink  = 1.0;
    if isfield(context,'shrink')
        shrink =context.shrink;
    end
    if (shrink==1)
        nn =max(max(conns));
        x = gather(context.x, 1:nn, 'values', 'noreshape'); % coordinates of nodes
        u = gather(context.u, 1:nn, 'values', 'noreshape'); % coordinates of nodes
        xu=x+u;
        if (isfield(context,'colorfield'))
            colors =gather(context.colorfield, 1:nn, 'values', 'noreshape'); % coordinates of nodes
            options=struct ('colors',colors);
        end
        if (isfield(context,'edgecolor'))
            options.edgecolor =context.edgecolor;
        end
        edgecolor='black';
        options.edgecolor =edgecolor;
        if ~strcmp(edgecolor,'none')
            for i=1:size(ef,1)
                draw_polyline (gv, xu, conns(:,ef(i,:)), options);
            end
        end
        options.edgecolor ='none';
        for i=1:size(f,1)
            draw_polygon (gv, xu, conns(:,f(i,:)), options);
        end
    else
        for ac=1:size(conns,1)
            conn =conns(ac,:);
            x = gather(context.x, conn, 'values', 'noreshape'); % coordinates of nodes
            u = gather(context.u, conn, 'values', 'noreshape'); % coordinates of nodes
            if (isfield(context,'colorfield'))
                colors =gather(context.colorfield, conn, 'values', 'noreshape'); % coordinates of nodes
                options=struct ('colors',colors);
            end
            xu=x+u;
            if isfield(context,'shrink') & context.shrink~=1
                pc= [-1,-1,-1;
            1,-1,-1;
            1,1,-1;
            -1,1,-1;-1,-1,1;
            1,-1,1;
            1,1,1;
            -1,1,1;0,-1,-1;
            1,0,-1;
            0,1,-1;
            -1,0,-1;0,-1,1;
            1,0,1;
            0,1,1;
            -1,0,1;-1,-1,0;
            1,-1,0;
            1,1,0;
            -1,1,0;0,0,-1;
            0,-1,0;
            1,0,0;
            0,1,0;
            -1,0,0;
            0,0,1;0,0,0];
                n=27;
                c=ones(n,1)*(sum(pc)/n);
                pc=c+context.shrink*(pc-c);
                xus=xu;
                for j=1:n
                    N=bfun(self,pc(j,:));
                    xus(j,:)=N'*xu;
                end
                xu=xus;
            end
            options.edgecolor =edgecolor;
            if ~strcmp(edgecolor,'none')
                for i=1:size(ef,1)
                    draw_polyline (gv, xu, ef(i,:), options);
                end
            end
            options.edgecolor ='none';
            for i=1:size(f,1)
                draw_polygon (gv, xu, f(i,:), options);
            end
        end;
    end
end
