% Constructor  for diffusion properties of materials.
%
% function retobj = property_diffusion (varargin)
%
% This class represents diffusion properties of materials.
% See discussion of constructors in <a href="matlab:helpwin 'FAESOR/classes/Contents'">classes/Contents</a>. 
% Parameters:
%     conductivity=matrix of conductivities; 2x2 for 2-D domains, 3x3
%                  for 3-D domains; the components are given in the local
%                  coordinate frame (feblock decides what that frame is)
%     specific_heat=specific heat per unit volume
%     source=source of heat: function handle or a constant
%     rho=mass density
%
function retobj = property_diffusion (varargin)
    class_name='property_diffusion';
    parent=property;
    if (nargin == 0)
        self.conductivity = 1.0;
        self.specific_heat= 1.0;
        self.source = [];
        retobj = class(self,class_name, parent);
        return;
    elseif (nargin == 1)
        arg = varargin{1};
        if strcmp(class(arg),class_name) % copy constructor.
            retobj = arg;
            return;
        else
            Parameters =varargin{1};
            self.conductivity = Parameters.conductivity;
            if (det(self.conductivity) < 0)
                error('Negative determinant of conductivity!');
            end
            specific_heat= 1.0;
            if isfield(Parameters,'specific_heat')
                specific_heat=Parameters.specific_heat;
            end
            self.specific_heat = specific_heat;
            source = [];
            if isfield(Parameters,'source')
                source=Parameters.source;
            end
            self.source = source;
            retobj = class(self,class_name, parent);
            return;
        end
    else
        error('Illegal arguments!');
    end
    return;
end
