% Advance the state by specified number of steps
%   Call as:
%      advance(obj, torque, dt, nsteps)
%   where
%      obj       - object
%      torque    - handle to function defining spatial torque
%      dt        - time step
%      nsteps    - number of steps to take
%
%   The first form returns the list of property names as a string,
%   the second form returns the value of the named property.
%
function retobj = advance(self,torque,dt,nsteps)
    tol=100*eps;
    Omegan1 = self.Omega;
    Rn1 = self.R;
    t=get(self.rotint,'t');
    I=self.I;
    for i=1:nsteps
        Omegamid=slvimpl(I, t, dt, Rn1, I*Omegan1, torque, tol);
        Rn=(Rn1*reortho(rotmat(dt*Omegamid)));
        Rmid=Rn1*rotmat(dt/2*Omegamid);
        Tmid=Rmid'*feval(torque, t+dt/2, Rmid);
        Omegan = Omegan1+I\(-dt*skewmat(Omegamid)*I*Omegamid+dt*Tmid);
        % swap variables for next step
        Rn1=Rn;
        Omegan1=Omegan;
        t=t+dt;
    end
    self.R = Rn;
    self.Omega = Omegan;
    self.rotint = set(self.rotint,'t',t);
    retobj=self;
end

function Omegamid=slvimpl(I, t, dt, Rn1, Pin1, torque, ceps)
    Omegamid=[0;0;0];
    i=0; maxi=50;
    while (1)
        pOmegamid=Omegamid;
        Rmid=Rn1*rotmat(dt/2*Omegamid);
        Tmid=Rmid'*feval(torque, t+dt/2, Rmid);
        Omegamid=I\(Pin1-dt/2*skewmat(Omegamid)*I*Omegamid+dt/2*Tmid);
        i=i+1;
        if (norm(pOmegamid-Omegamid) < ceps)
            break;
        end
        if (i > maxi)
            if (norm(pOmegamid-Omegamid) > 1e-9)
                warning(['imid - Failed to converge: ||residual||=' num2str(norm(pOmegamid-Omegamid))]);
            end
            break;
        end
    end
    return;
end
