%clear all
kappa_concrete=1.81*[1 0; 0 1]; % W/K/m
rho_concrete = 2350;% kg/m^3
cv_concrete =0.22*rho_concrete;
Ti = 19;% in degree Celsius
Ta_inner=[19, 18, 17, 18, 20, 26, 30, 30, 29, 26, 22, 20];
Ta_inner= [Ta_inner Ta_inner Ta_inner Ta_inner Ta_inner];
Ta_lower=[19, 18, 17, 25, 28, 35, 32, 31, 32, 27, 22, 20];
Ta_lower= [Ta_lower Ta_lower Ta_lower Ta_lower Ta_lower];
Ta_upper=[19, 18, 17, 35, 65, 80, 80, 60, 45, 35, 22, 20];
Ta_upper= [Ta_upper Ta_upper Ta_upper Ta_upper Ta_upper];
Ta_vert=[19, 18, 17, 52, 65, 56, 35, 31, 32, 27, 22, 20];
Ta_vert= [Ta_vert Ta_vert Ta_vert Ta_vert Ta_vert];
hs=[6.2, 6.2, 6, 6.3, 7.0, 7.3, 9.5, 13.4, 17.1, 17.6, 15.4, 9.5];
hs= [hs hs hs hs hs];
%
times =(0:2:2*(length(hs)-1))*3600 ; % in seconds;
dt=1*3600; % time  increment in seconds
tend= (4*24+16)*3600; % length of the time interval
t=0;
crank.hot = 'co-'; crank.cold ='m*-'; crank.theta = 0.5;
bweul.hot = 'bo-'; bweul.cold ='r*-'; bweul.theta = 1.0;
settings =bweul;
theta = settings.theta; 
online_graphics=~~true;% plot the solution as it is computed?
mesh_size=0.1; %
[fens,gcells,groups,edge_gcells,edge_groups]=targe2_mesher({...
    'curve 1 line 0 0 2.60 0',...
    'curve 2 line 2.60 0 2.6 2.00',...
    'curve 3 line 2.6 2.0 4.85 2.0',...
    'curve 4 line 4.85 2 4.85 2.2',...
    'curve 5 line 4.85 2.2 0.0 2.2',...
    'curve 6 line 0.0 2.2 0.0 2',...
    'curve 7 line 0 2 2.1 2',...
    'curve 8 line 2.1 2 2.1 0.2',...
    'curve 9 line 2.1 0.2 0.0 0.2',...
    'curve 10 line 0 0.2 0 0',...
    ['subregion 1  property 1 boundary '...
    ' 1 2 3 4 5 6 7 8 9 10 '],...
    ['m-ctl-point constant ' num2str(mesh_size)]
    }, 1.0);
prop_concrete=...
    property_diffusion (struct('conductivity',kappa_concrete,...
    'specific_heat',cv_concrete,'source',0.0));
mater_concrete=mater_diffusion (struct('property',prop_concrete));
feb_concrete = feblock_diffusion (struct ('mater',mater_concrete,...
    'gcells',subset(gcells,groups{1}),...
    'integration_rule',tri_rule(1)));
egcells_inner=subset(edge_gcells,[edge_groups{[(7:9)]}]);
egcells_lower=subset(edge_gcells,[edge_groups{[1, 3]}]);
egcells_upper=subset(edge_gcells,[edge_groups{[5]}]);
egcells_vert=subset(edge_gcells,[edge_groups{[2, 4]}]);
geom = field(struct('name',['geom'], 'dim', 2, 'fens',fens));
tempn=field(struct('name',['tempn'], 'dim', 1, 'nfens',...
    get(geom,'nfens')));
amb = clone(tempn, ['amb']);

tempn = numbereqns (tempn);
tempn = scatter_sysvec(tempn,gather_sysvec(tempn)*0+Ti);
C = start (sparse_sysmat, get(tempn, 'neqns'));
C = assemble (C, capacity(feb_concrete, geom, tempn));
Cm = get(C,'mat');
K = start (sparse_sysmat, get(tempn, 'neqns'));
K = assemble (K, conductivity(feb_concrete, geom, tempn));
Kmc = get(K,'mat');

if online_graphics
    gv=graphic_viewer;
    gv=reset (gv,struct('limits',[0, 4.85, 0, 2.2, 0, 2.5]));
    dcm=data_colormap(struct('range',[20, 70],'colormap',jet));
end
minmaxT = []; ts = [];
t= 0; 
while t<tend+0.1*dt % Time stepping
    if online_graphics
        gv=reset (gv,struct('limits',[0, 4.85, 0, 2.2, 0, 3.5]));
        set(gca,'FontSize', 14)
        T=get(tempn,'values');
        colorfield=field(struct('name',['colorfield'],'data',...
            map_data(dcm, T)));
        geomT=field(struct ('name', ['geomT'], ...
            'data',[get(geom,'values'), 0.1*(get(tempn,'values')-Ti)]));
        draw(gcells,gv,struct('x',geomT, 'u',0*geomT,...
                'colorfield',colorfield, 'edgecolor','none'));
            draw(gcells,gv,struct('x',geom, 'u',0*geom, ...
                'facecolor','none'));
        camset(gv,[25.5545   11.6402   34.6031    1.9206    1.1491    1.5038   -0.1301    0.9681   -0.2140    6.0276])
        xlabel('X [m]');        ylabel('Y [m]');
        zlabel('Temperature [degrees C]')
        figure(gcf);
        hour =mod (round(t/3600), 24)
        day =floor(t/3600/24)+1
        title (['Day ' num2str(day) ', ' num2str(hour) ' hours']); hold off; pause(0.1);
        %         saveas(gcf, ['shrinkfit-' num2str(t) '.png'], 'png');
    end
    tempn1 = tempn;
    h=interp1(times,hs,t);
    efeb = feblock_diffusion (struct ('mater',mater_concrete,...
        'gcells',subset(edge_gcells,[edge_groups{[(1:5),(7:9)]}]),...
        'integration_rule',gauss_rule(1,1),...
        'surface_transfer', h));
    K = start (sparse_sysmat, get(tempn, 'neqns'));
    K = assemble (K, surface_transfer(efeb, geom, tempn));
    Km = Kmc+get(K,'mat');
    F = start (sysvec, get(tempn, 'neqns'));
    conn = connected_nodes(egcells_inner);
    Ta = interp1(times,Ta_inner,t);
    amb = set_ebc(amb, conn, conn*0+1, [], conn*0+Ta);
    conn = connected_nodes(egcells_lower);
    Ta = interp1(times,Ta_lower,t);
    amb = set_ebc(amb, conn, conn*0+1, [], conn*0+Ta);
    conn = connected_nodes(egcells_upper);
    Ta = interp1(times,Ta_upper,t);
    amb = set_ebc(amb, conn, conn*0+1, [], conn*0+Ta);
    conn = connected_nodes(egcells_vert);
    Ta = interp1(times,Ta_vert,t);
    amb = set_ebc(amb, conn, conn*0+1, [], conn*0+Ta);
    amb = apply_ebc (amb);
    F=assemble(F, surface_transfer_loads(efeb,geom,tempn,amb));
    Tn=gather_sysvec(tempn);
    minmaxT= [minmaxT; min(Tn),max(Tn)]
    ts= [ts,t];
    Tn1=(1/dt*Cm+theta*Km)\((1/dt*Cm-(1-theta)*Km)*Tn+get(F,'vec'));
    tempn = scatter_sysvec(tempn1,Tn1);
    t=t+dt;
%     if t>24*3*tmul
%         dt=0.1*tmul; % time 0
%     end
end
if online_graphics
    rotate(gv)
    figure
end
plot(ts/3600,minmaxT(:, 1),settings.hot,'linewidth', 2); hold on
plot(ts/3600,minmaxT(:, 2),settings.cold,'linewidth', 2)
set(gca,'FontSize', 14); grid on
xlabel('Time in hours')
ylabel('Temperature in degrees Celsius')
