disp('Example simple_blocker_nicer large strain nonlinear analysis; neo-hookean material');
% Parameters:
E=100;
nu=0.49993;
% nu=0.3;
ebcs=[];
Fmag=-0.5;
loaddir=1;
nincr = 12;
scale=2;
stressscale=scale/20;

% Mesh
% [fens,gcells] = t4blockdel(10,20,10, 3, 7, 3);
% ir = simplex_nq_rule (3);
% [fens,gcells] = block(10,20,10,  4, 8, 4);
% ir =tensprod_nq_rule(3);
% [fens,gcells] = block(10,20,10, 3, 5, 4);
[fens,gcells] = block(10,20,10, 2, 3, 2);
[fens,gcells] = H8_to_H27(fens,gcells);
ir =tensprod_nq_rule (struct('dim',3,'order',2));
% Material
prop = property_linel_iso (struct('E',E,'nu',nu));
mater = mater_defor_neohook(struct('property',prop));
% Finite element block
feb = feblock_defor_nonlinear (struct ('mater',mater, 'gcells',gcells, 'integration_rule',ir));
% feb = feblock_defor_nonlinear_nice (struct ('mater',mater,'gcells',gcells, 'integration_rule',ir));
% Geometry
geom = field(struct ('name',['geom'], 'dim', 3, 'fens',fens));
% Define the displacement field
u   = 0*geom; % zero out
% Apply EBC's

ebc_fenids=fenode_select (fens,struct('box',[0,100,0,0,0,100],'inflate',44/10000));
ebc_prescribed=ones(1,length (ebc_fenids));
ebc_comp=[];
ebc_val=ebc_fenids*0;
u   = set_ebc(u, ebc_fenids, ebc_prescribed, ebc_comp, ebc_val);
u   = apply_ebc (u);

% Number equations
u   = numbereqns (u);
% Assemble the system matrix
ems = stiffness(feb, geom, u,u);
K = start (sparse_sysmat, get(u, 'neqns'));
K = assemble (K, ems);

% %linear case (nicer)
% propm = property_linel_iso(struct('E',E,'nu', nu));
% feblin=feblock_defor_ss_nicer(struct ('mater',mater_defor_ss_linel_triax (struct ('property',propm)), 'gcells',gcells, 'integration_rule',tensprod_nq_rule(struct ('dim',3,'order',1))));
% ems = stiffness(feblin, geom, u);
% Klin = start (dense_sysmat, get(u, 'neqns'));
% Klin = assemble (Klin, ems);
% 
% %linear case (nice)
% propm = property_linel_iso(struct('E',E,'nu', nu));
% feblin=feblock_defor_ss_nice(struct ('mater',mater_defor_ss_linel_triax (struct ('property',propm)), 'gcells',gcells, 'integration_rule',tensprod_nq_rule(3)));
% ems = stiffness(feblin, geom, u);
% Klin1 = start (dense_sysmat, get(u, 'neqns'));
% Klin1 = assemble (Klin1, ems);

% Load
Load=zeros(3,1); Load(loaddir) =Fmag*1.0;
fi=force_intensity(struct('magn',Load));
F = start (sysvec, get(u, 'neqns'));
F = assemble (F, distrib_loads(feb, geom, u, fi, 3));
% Solve
a = get(K,'mat');
b = get(F,'vec');
x = a\b;
u = scatter_sysvec(u, x);
% Plot
gv=graphic_viewer;
gv=reset(clear(gv,[]),[]);
draw(feb,gv, struct ('x', geom,'u',scale*u, 'facecolor','red'));
disp('Done with linear solution: hit return'); % pause


ulin = u; %  linear displacement solution

% Now comes the nonlinear solution
tup = 1;
u = u*0; % zero out the displacement
utol = 1e-12;

t=0; % time=load magnitude
incr=1;
while (incr <= nincr)
    t = t + tup / nincr;
    % Initialization
    u1 = u; % guess
    u1 = apply_ebc(u1);
    du = 0*u; % this will hold displacement increment
    du = apply_ebc(du);
    iter=1;
    gv=reset(clear(gv,[]),[]);
    draw(feb,gv, struct ('x', geom,'u',u, 'facecolor','red'));
    while 1
        feb1=feb;                                           % Make a copy of the state
        Load=zeros(3,1); Load(loaddir) =Fmag*t;
        fi=force_intensity(struct('magn',Load));
        F = start (sysvec, get(u, 'neqns'));
        F = assemble (F, distrib_loads(feb, geom, u, fi, 3));
        [feb1,evs] = restoring_force(feb1,geom,u1,u);       % Internal forces
        F = assemble (F, evs);
        F = finish(F);
        K = start (sparse_sysmat, get(u, 'neqns'));
        ems=stiffness(feb1, geom, u1,u);% Current material stiffness
        ems =cat(2,ems,stiffness_geo(feb1, geom, u1,u)); % Current geometric stiffness
        K = assemble (K, ems);  
        % Displacement increment
        x = K\F;
%                 x =  (get(K,'mat')+get(K,'mat'))/2 \ get(F,'vec');;
        du = scatter_sysvec(du, x);
        u1 = u1 + du;                                       % increment displacement
        draw(feb1,gv, struct ('x', geom,'u',u1, 'facecolor','none'));
        disp(['   It. ' num2str(iter) ': ||du||=' num2str(norm(du))]);
        if (norm(du) < utol) break; end;                    % convergence check
        pause(1)
        iter=iter+1;
    end
    [feb,ignore] = restoring_force(feb,geom,u1,u);        % final update
    disp(['Increment ' num2str(incr) ': Converged for t=' num2str(t)]); % pause
    u = u1;                                               % update the displacement
    gv=reset(clear(gv,[]),[]);
    draw(feb,gv, struct ('x', geom,'u',u, 'facecolor','red'));
    draw(feb,gv, struct ('x', geom,'u',0*ulin, 'facecolor','none'));
    incr = incr + 1;
end
