function topbeam
disp('Example topbeam: small strain nonlinear analysis');
% Parameters:
E=1000;
nu=0.3;
sigma_y=2;
L= 8; % Length of the beam
W = 2; % width
H = 2; % height
magn=3/50;
scale=15;
stressscale=scale/20;
epscale=50;
nincr = 1;


% Mesh
% [fens,gcells] = blockx([0, 0.5, 1, 2, 4, 6, 7, 7.5, 8]/8*L,[0,1,2]/2*W/2,[0,1,2,3]/3*H);
% [fens,gcells] = blockx([0,  1, 2, 6, 7, 8]/8*L,[0,2]/2*W/2,[0,1,2,3]/3*H);
[fens,gcells] = blockx([0,  8]/8*L,[0,2]/2*W/2,[0,2]/2*H);
% [fens,gcells] = H8_to_H20(fens,gcells);
% Material
prop = property_plastic_perf_j2_iso(struct('E',E,'nu',nu,'sigma_y',sigma_y));
mater = mater_ss_plastic_perf_j2_triax(struct('property',prop));
% Finite element block
feb = feblock_defor_ss_nonlinear (struct ('mater',mater, 'gcells',gcells,...
    'integration_rule',gauss_rule(3,2)));
% Geometry
geom = field(struct ('name',['geom'], 'dim', 3, 'fens',fens));
% Define the displacement field
u   = 0*geom; % zero out
Center_fenids=fenode_select (fens,struct('box',[L,L,W/2, W/2,-Inf,Inf],'inflate',1/1000));
% Apply EBC's
ebc_fenids=fenode_select (fens,struct('box',[0,0,-Inf,Inf,-Inf,Inf],'inflate',1/1000));
ebc_prescribed=ones(1,length (ebc_fenids));
ebc_comp=[];
ebc_val=ebc_fenids*0;
u   = set_ebc(u, ebc_fenids, ebc_prescribed, ebc_comp, ebc_val);
ebc_fenids=fenode_select (fens,struct('box',[L,L,-Inf,Inf,-Inf,Inf],'inflate',1/1000));
ebc_prescribed=ones(1,length (ebc_fenids));
ebc_comp=ones(1,length (ebc_fenids))*1;
ebc_val=ebc_fenids*0;
u   = set_ebc(u, ebc_fenids, ebc_prescribed, ebc_comp, ebc_val);
ebc_fenids=fenode_select (fens,struct('box',[-Inf,Inf,W/2, W/2,-Inf,Inf],'inflate',1/1000));
ebc_prescribed=ones(1,length (ebc_fenids));
ebc_comp=ones(1,length (ebc_fenids))*2;
ebc_val=ebc_fenids*0;
u   = set_ebc(u, ebc_fenids, ebc_prescribed, ebc_comp, ebc_val);
u   = apply_ebc (u);
% Number equations
u   = numbereqns (u);
% Assemble the system matrix
ems = stiffness(feb, geom, u, u);
K = start (dense_sysmat, get(u, 'neqns'));
K = assemble (K, ems);
% Load
bdry_gcells = mesh_bdry(gcells, []);
bcl = gcell_select(fens, bdry_gcells, ...
    struct ('box',[-Inf,Inf,-Inf,Inf,H H],'inflate',H/1000));
lfeb = feblock_defor_ss(struct ('mater',mater, 'gcells',bdry_gcells(bcl),...
    'integration_rule',gauss_rule(2,2)));
fi= force_intensity(struct ('magn',[0;0; magn]));
F = start (sysvec, get(u, 'neqns'));
F = assemble (F, distrib_loads(lfeb, geom, u, fi,2));
% Solve
a = get(K,'mat');
b = get(F,'vec');
x = a\b;
u = scatter_sysvec(u, x);


ulin = u; %  linear displacement solution

% Now comes the nonlinear solution
tup = 1;
u = u*0; % zero out the displacement
utol = 10e-6;

gv=graphic_viewer;

Center_deflection = [];
t=0; % time=load magnitude
incr=1;
while (incr <= nincr)
    t = t + tup / nincr;
    % Initialization
    u1 = u; % guess
    u1 = apply_ebc(u1);
    du = 0*u; % this will hold displacement increment
    du = apply_ebc(du);
    iter=1;
    du0norm = [];
    while 1
        feb1=feb;                                           % Make a copy of the state
        fi= force_intensity(struct ('magn',[0;0; t*magn]));
        F = start (sysvec, get(u, 'neqns'));
        F = assemble (F, distrib_loads(lfeb, geom, u1, fi,2));
        [Ignore,evs] = restoring_force(feb1,geom,u1,u);       % Internal forces
        F = assemble (F, evs);
        K = start (sparse_sysmat, get(u, 'neqns'));
%         K = assemble (K, ...
%             cat(2,stiffness(feb1, geom, u1,u), stiffness_geo(feb1, geom, u1,u)));               % Current geometric stiffness
        K = assemble (K, stiffness(feb1, geom, u1,u));               % Current geometric stiffness
        % Displacement increment
        du = scatter_sysvec(du, K\F);
        u1 = u1 + du;                                       % increment displacement
        dunorm =norm(du);
        if (isempty(du0norm)) du0norm =dunorm;end
        disp(['   Iteration ' num2str(iter) ': ||du||/||du0||=' num2str(dunorm/du0norm)]);
        %
        gv=reset(clear(gv,[]),[]);
        draw(feb1,gv, struct ('x', geom,'u',scale*u1, 'facecolor','none'));
        id.comp= 1;
        id.container=-Inf;
        id=inspect_integration_points(feb1, geom, u1, [],...
            (1:length (gcells)), struct ('output',['equiv_pl_def']),...
            @mx,id);
        max_equiv_pl_def=id.container
        id.container=Inf;
        id=inspect_integration_points(feb1, geom, u1, [], ...
            (1:length (gcells)), struct ('output',['equiv_pl_def']),...
            @mn,id);
        min_equiv_pl_def =id.container
        dcm=data_colormap(struct ('range',[min_equiv_pl_def,max_equiv_pl_def], 'colormap',jet));
        draw_integration_points(feb1,gv,struct ('x',geom,'u',u1,'u_displ',scale*u1, 'scale',epscale,'quantity',['equiv_pl_def'],'component',1,'data_cmap', dcm));
        pause(1)
        %
        Center_u =gather (u,Center_fenids,'values','noreshape');

        if (dunorm < utol*du0norm) break; end;                    % convergence check
        iter=iter+1;
    end
    [feb,ignore] = restoring_force(feb1,geom,u1,u);        % final update
    u = u1;                                               % update the displacement
    gv=reset(clear(gv,[]),[]);
    draw(feb,gv, struct ('x', geom,'u',scale*u, 'facecolor','none'));
    id.comp= 1;
    id.container=-Inf;
    id=inspect_integration_points(feb, geom, u, [],...
        (1:length (gcells)), struct ('output',['equiv_pl_def']),...
        @mx,id);
    max_equiv_pl_def=id.container
    id.container=Inf;
    id=inspect_integration_points(feb, geom, u, [], ...
        (1:length (gcells)), struct ('output',['equiv_pl_def']),...
        @mn,id);
    min_equiv_pl_def =id.container
    dcm=data_colormap(struct ('range',[min_equiv_pl_def,max_equiv_pl_def], 'colormap',jet));
    draw_integration_points(feb,gv,struct ('x',geom,'u',u,'u_displ',scale*u, 'scale',epscale,'quantity',['equiv_pl_def'],'component',1,'data_cmap', dcm));
    pause(1)
    disp(['Increment ' num2str(incr) ': Converged for t=' num2str(t) ]); % pause
    Center_deflection = [Center_deflection,mean(Center_u(:,3))];
    incr = incr + 1;
end
Center_deflection
end

function id= mn(id,out,xyz,pc)
id.container=min(out(id.comp), id.container);
end

function id= mx(id,out,xyz,pc)
id.container=max(out(id.comp), id.container);
end
%draw(gmlin, 'dataidx', 3, 'scale', scale, 'displace_by', [1 2 3], 'facecolor', 'none')