% Time-dependent heating of tissue. The electrical conductivity is a function of temperature.
% Adaptive time stepping algorithm is used.
function Simple_unipolar
    
    Number_of_refinements = 1;
    
    xj=[1;1;1;2;2;4];
    m=0*xj+Number_of_refinements;
    xe1 = [0,sum(xj(1:2))];
    xe2 = [0,sum(xj(1:end))];
    xs=[0];
    for k= 1:length(xj)
        xs=[xs,linspace(xs(end),xs(end)+xj(k),m(k)+1)];
    end
    xs=unique(xs);
    
    yj=xj;
    n=m;
    ye1 = [0,sum(yj(1:2))];
    ye2 = [0,sum(yj(1:end))];
    ys=[0];
    for k= 1:length(yj)
        ys=[ys,linspace(ys(end),ys(end)+yj(k),n(k)+1)];
    end
    ys=unique(ys);
    
    zj=[3;1;1];
    p=0*zj+Number_of_refinements;
    ze1 = [sum(zj(1:end)),sum(zj(1:end))];
    ze2 = [0, 0];
    zd = cumsum(zj);
    zs=[0];
    for k= 1:length(zj)
        zs=[zs,linspace(zs(end),zs(end)+zj(k),p(k)+1)];
    end
    zs=unique(zs);
    
    [fens,gcells] = H8_blockx(xs,ys,zs);
    [fens,gcells] = H8_to_H27(fens,gcells);
    
    tol=min([min(diff(xs)),min(diff(ys)),min(diff(zs))])/100;
    
    % ir3d=tensprod_nq_rule(struct('dim',3,'order',2));
    % ir2d=tensprod_nq_rule(struct('dim',2,'order',2));
    ir3d=gauss_rule(3, 3);
    ir2d=gauss_rule(2, 3);
    
    % bg=mesh_bdry(gcells);
    % drawmesh({fens,bg},'gcells','facecolor','red');
    
    Voltage1 = 9;% V
    Voltage2 = -9;% V
    Ti=37;% degree Celsius
    rho_tissue=1100*1e-9;% 1100 kg/m^3
    cv_tissue=3111*rho_tissue;% 3111 J/kg/K
    k_tissue=0.531*1e-3;% 0.531 W/m/K
    h_tistrunc=200*1e-6;% 200 W/m^2/K
    h_blood=2000*1e-6;% 2000 W/m^2/K
    
    postfile =[mfilename '-' num2str(count(gcells))];
    
    disp (['Postprocessing file: ' postfile])
    
    dt=0.05;
    tend=160;
    tolT=0.2;
    tolTp=4;
    maxit=3;
    
    thermal_prop=property_thermal_diffusion(struct('thermal_conductivity',k_tissue*eye(3),...
        'specific_heat',cv_tissue));
    thmater=mater_diffusion (struct('property',thermal_prop));
    electrical_prop=property_electrostatics(struct('electrical_conductivity',@(T)(0.0002*(1+0.02*(T-37))*eye(3))));
    elmater=mater_electrostatics (struct('property',electrical_prop));
    elfeb = feblock_electrostatics (struct ('mater',elmater,...
        'gcells',gcells,'integration_rule',ir3d));
    thfeb = feblock_diffusion (struct ('mater',thmater,...
        'gcells',gcells,'integration_rule',ir3d));
    bgcells=mesh_bdry(gcells);
    
    bgcellslbot = gcell_select(fens, bgcells, struct('box',[0,xs(end), 0,ys(end),0,0],...
        'inflate', tol));
    bgcellsltop = gcell_select(fens, bgcells, struct('box',[0,xs(end), 0,ys(end),zs(end),zs(end)],...
        'inflate', tol));
    bgcellslsides = [gcell_select(fens, bgcells, struct('box',[xs(end),xs(end), 0,ys(end),0,zs(end)],...
        'inflate', tol)),...
        gcell_select(fens, bgcells, struct('box',[0,xs(end), ys(end),ys(end),0,zs(end)],...
        'inflate', tol))];
    elfebbot = feblock_electrostatics (struct ('mater',elmater,...
        'gcells',subset(bgcells,bgcellslbot),...
        'integration_rule',ir2d));
    thfebbot = feblock_diffusion (struct ('mater',thmater,...
        'gcells',subset(bgcells,bgcellslbot),...
        'integration_rule',ir2d,...
        'surface_transfer', h_tistrunc));
    elfebtop = feblock_electrostatics (struct ('mater',elmater,...
        'gcells',subset(bgcells,bgcellsltop),...
        'integration_rule',ir2d));
    thfebtop = feblock_diffusion (struct ('mater',thmater,...
        'gcells',subset(bgcells,bgcellsltop),...
        'integration_rule',ir2d,...
        'surface_transfer', h_blood));
    elfebsides = feblock_electrostatics (struct ('mater',elmater,...
        'gcells',subset(bgcells,bgcellslsides),...
        'integration_rule',ir2d,...
        'surface_transfer', h_tistrunc));
    thfebsides = feblock_diffusion (struct ('mater',thmater,...
        'gcells',subset(bgcells,bgcellslsides),...
        'integration_rule',ir2d,...
        'surface_transfer', h_tistrunc));
    % bg=mesh_bdry(gcells);
    % drawmesh({fens,bgcells(bgcellsltop)},'gcells','facecolor','blue');
    % drawmesh({fens,bgcells(bgcellslbot)},'gcells','facecolor','green');
    % drawmesh({fens,bgcells(bgcellslsides)},'gcells','facecolor','red');
    geom = field(struct('name',['geom'], 'dim', 3, 'fens',fens));
    theta=field(struct('name',['theta'], 'dim', 1, 'nfens',get(geom,'nfens')));
    phi=field(struct('name',['phi'], 'dim', 1, 'nfens',get(geom,'nfens')));
    
    fenids=fenode_select(fens,struct('box',[xe1,ye1,ze1],...
        'inflate', tol)) ; % prescribed Voltage on Electrode 1
    prescribed=fenids*0+1;
    comp=[];
    val=zeros(length(fenids),1)+Voltage1;
    phi = set_ebc(phi, fenids, prescribed, comp, val);
    phi = apply_ebc (phi);
    fenids=fenode_select(fens,struct('box',[xe2,ye2,ze2],...
        'inflate', tol)) ; % prescribed Voltage on Electrode 2
    prescribed=fenids*0+1;
    comp=[];
    val=zeros(length(fenids),1)+Voltage2;
    phi = set_ebc(phi, fenids, prescribed, comp, val);
    
    phi = apply_ebc (phi);
    phi = numbereqns (phi);
    
    theta = numbereqns (theta);
    % Initial temperature
    theta = scatter_sysvec(theta,gather_sysvec(theta)*0+Ti);
    ambtheta=theta;% ambient temperature
    egcells=subset(bgcells,bgcellslbot);
    conn = connected_nodes(egcells);
    ambtheta = set_ebc(ambtheta, conn, conn*0+1, [], conn*0+Ti);
    egcells=subset(bgcells,bgcellsltop);
    conn = connected_nodes(egcells);
    ambtheta = set_ebc(ambtheta, conn, conn*0+1, [], conn*0+Ti);
    egcells=subset(bgcells,bgcellslsides);
    conn = connected_nodes(egcells);
    ambtheta = set_ebc(ambtheta, conn, conn*0+1, [], conn*0+Ti);
    
    
    C = start (sparse_sysmat, get(theta, 'neqns'));
    C = assemble (C, capacity(thfeb, geom, theta));
    Cm = get(C,'mat');
    clear C
    Kt = start (sparse_sysmat, get(theta, 'neqns'));
    Kt = assemble (Kt, conductivity(thfeb, geom, theta));
    Ktm = get(Kt,'mat');
    Kt = start (sparse_sysmat, get(theta, 'neqns'));
    Kt = assemble (Kt, cat(2,surface_transfer(thfebbot, geom, theta),...
        surface_transfer(thfebtop, geom, theta),...
        surface_transfer(thfebsides, geom, theta)...
        ));
    Ktm = Ktm+get(Kt,'mat');
    clear Kt
    
    % Initial distribution of the electrical field
    Ke = start (sparse_sysmat, get(phi, 'neqns'));
    Ke = assemble (Ke, electrical_conductivity(elfeb, geom, phi, theta));
    Fe = start (sysvec, get(phi, 'neqns'));
    Fe = assemble (Fe, nz_ebc_loads_electrical_conductivity(elfeb, geom, phi, theta));
    phi = scatter_sysvec(phi, Ke\Fe);
    
    t= 0;
    snapshot=1;
    ts{snapshot} = t;
    phis{snapshot} = get(phi,'values');
    thetas{snapshot} = get(theta,'values');
    timer=timetic;
    disp([num2str(snapshot) ' Time ' num2str(t)])
    Tn = gather_sysvec(theta);
    pdt=dt;
    while t<tend % Time stepping
        tic(timer);
        pTn1=gather_sysvec(theta);
        Tnp  = pTn1 +dt/pdt*(pTn1-Tn);
        for i=1:maxit
            Ke = start (sparse_sysmat, get(phi, 'neqns'));
            Ke = assemble (Ke, electrical_conductivity(elfeb, geom, phi, theta));
            Fe = start (sysvec, get(phi, 'neqns'));
            Fe = assemble (Fe, nz_ebc_loads_electrical_conductivity(elfeb, geom, phi, theta));
            phi = scatter_sysvec(phi,  Ke\Fe);
            clear Ke
            
            Ft = start (sysvec, get(theta, 'neqns'));
            Ft = assemble (Ft, resistive_heating_loads(elfeb, geom, theta, phi));
            Fth = start (sysvec, get(theta, 'neqns'));
            Fth=assemble(Fth, surface_transfer_loads(thfebtop,geom,theta,ambtheta));
            Fth=assemble(Fth, surface_transfer_loads(thfebbot,geom,theta,ambtheta));
            Fth=assemble(Fth, surface_transfer_loads(thfebsides,geom,theta,ambtheta));
            Fv=get(Fth,'vec')+get(Ft,'vec');
            Tn1=(1/dt*Cm+Ktm)\((1/dt*Cm)*Tn+Fv);
            theta = scatter_sysvec(theta, Tn1);
            nT=norm(Tn1-pTn1,Inf);
            disp(['  Max T=' num2str(max(Tn1)) ', norm(Tn1-pTn1)=' num2str(nT)]);
            if (nT<tolT)
                break;
            end
            pTn1 =Tn1;
        end
        Tn =Tn1;
        t=t+dt;
        snapshot=snapshot+1;
        ts{snapshot} = t;
        phis{snapshot} = get(phi,'values');
        thetas{snapshot} = get(theta,'values');
%         save(postfile, 'postfile', 'ts', 'phis', 'thetas');
        disp([num2str(snapshot) ' Time ' num2str(t) ' [CPU  ' num2str(toc(timer)) ']'])
        pdt = dt;
        if (~isempty( Tnp )) && (tolTp~= 0)
            disp(['  tolTp ' num2str(tolTp) ' norm(Tn1-Tnp,Inf)  ' num2str(norm(Tn1-Tnp,Inf)) ''])
            dt =dt*sqrt(tolTp/norm(Tn1-Tnp,Inf));
        end
        if ((tend-t<dt) && (t<tend)), dt=tend-t; end
    end
    
    bgcells=mesh_bdry(gcells);
    febsurf = feblock (struct ('gcells',bgcells,'integration_rule',ir2d));
    
    RangeT = [Inf,-Inf];
    RangeV = [Inf,-Inf];
    for snapshot = 1:length(ts)
        RangeT(1) =min( [RangeT(1),thetas{snapshot}']);
        RangeT(2) =max( [RangeT(2),thetas{snapshot}']);
        RangeV(1) =min( [RangeV(1),phis{snapshot}']);
        RangeV(2) =max( [RangeV(2),phis{snapshot}']);
    end
    
    gv=graphic_viewer;
    
    isovalues=[40,45,50,55];
    snapshot=1;
    for t= [ts{:}]
        phi = scatter (phi, (1:count(fens)),  phis{snapshot});
        theta = scatter (theta, (1:count(fens)),  thetas{snapshot});
        
        gv=reset (gv,[]);
        camset(gv,[-68.9305 -124.7078   61.4020    6.0174 10.5018   -1.0574         0         0  1.0000    3.7801]);
        dcm=data_colormap(struct('range',RangeT,'colormap',hot(12)));
        colorfield=field(struct ('name', ['colorfield'], 'data', map_data(dcm, get(theta,'values'))));
        camset(gv,[-68.9305 -124.7078   61.4020    6.0174 10.5018   -1.0574         0         0  1.0000    3.7801]);
        draw(febsurf, gv, struct ('x',geom, 'u',0*geom,'colorfield',colorfield,'facealpha', 0.3));
        for isovalue=isovalues
            options=struct ('x',geom, 'u',0*geom,'scalarfield',theta,...
                'isovalue',isovalue,'color',map_data(dcm, isovalue));
            draw_isosurface(gcells, gv, options);
        end
        figure(gcf); headlight(gv); pause(1)
%         saveas(gcf, [postfile '-' 'Tiso'  '-' n2s0p(snapshot) '.png'], 'png');
        
        snapshot=snapshot+1;
    end
    
    
    isovalues=linspace(-9, 9, 19 );
    snapshot=1;
    for t= [ts{:}]
        phi = scatter (phi, (1:count(fens)),  phis{snapshot});
        theta = scatter (theta, (1:count(fens)),  thetas{snapshot});
        
        gv=reset (gv,[]);
        camset(gv,[-68.9305 -124.7078   61.4020    6.0174 10.5018   -1.0574         0         0  1.0000    3.7801]);
        dcm=data_colormap(struct('range',RangeV,'colormap',jet(12)));
        colorfield=field(struct ('name', ['colorfield'], 'data', map_data(dcm, get(phi,'values'))));
        camset(gv,[-68.9305 -124.7078   61.4020    6.0174 10.5018   -1.0574         0         0  1.0000    3.7801]);
        draw(febsurf, gv, struct ('x',geom, 'u',0*geom,'colorfield',colorfield,'facealpha', 0.3));
        for isovalue=isovalues
            options=struct ('x',geom, 'u',0*geom,'scalarfield',phi,...
                'isovalue',isovalue,'color',map_data(dcm, isovalue));
            draw_isosurface(gcells, gv, options);
        end
        figure(gcf); headlight(gv); pause(1)
%         saveas(gcf, [postfile '-' 'Tiso'  '-' n2s0p(snapshot) '.png'], 'png');
        
        snapshot=snapshot+1;
    end
end