% Extrude a mesh of L2 line segments into a mesh of Q4 quadrilaterals.
%
% function [fens,gcells] = L2_extrude(fens,gcells,nLayers,extrusionh)
%
%
% Inputs:
% fens,gcells = nodes and cells of the quadrilateral mesh,
% nLayers= number of layers,
% extrusionh= handle to function that computes the extrusion vector
%   nxyz=extrusionh(xyz,k)
% 
% Output:
% fens= node set which incorporates the nodes on input plus the nodes of
% the extruded elements. Note that no merging (fusing) of nodes is performed.
% q4gcells = Q4 extruded geometric cells
function [q4fens,q4gcells] = L2_extrude(fens,gcells,nLayers,extrusionh)
    id = get(fens,'id');
    cn=connected_nodes(gcells);
    id(cn)=(1:length(cn))';
    L2gcells =update_connectivity(gcells,id);
    xyz=get(fens,'xyz');
    L2fens =fenodeset(struct('id',(1:length(cn))','xyz',xyz(cn,:)));
    
    [q4fens,q4gcells]= do_extrude(L2fens,L2gcells,nLayers,extrusionh);
%     [fens,efens,fens2] = fuse_nodes(efens, fens, 0);
%     id = get(efens,'id');
%     q4gcells =update_connectivity(egcells,id);
end

function [efens,egcells]= do_extrude(fens,gcells,nLayers,extrusionh)
     nn1 =count(fens);
    nnt=nn1*nLayers;
    ngc=count(gcells)*nLayers;
    xyz1=get(fens,'xyz');
    x1=extrusionh(xyz1(1,:),0)
    xyz =zeros(size(xyz1,1)*(nLayers+1),size(x1,2));
    qconn=zeros(ngc,4);
    for j=1:nn1
        xyz(j,:) =extrusionh(xyz1(j,:),0);
    end
    for k=1:nLayers
        for j=1:nn1
            f=j+k*nn1;
            xyz(f,:) =extrusionh(xyz1(j,:),k);
        end
    end
    
    conn = get(gcells,'conn');
    gc=1;
    for k=1:nLayers
        for i=1:count(gcells)
            qconn(gc,:)=[conn(i,:)+(k-1)*nn1, conn(i,end:-1:1)+k*nn1];
            gc=gc+1;
        end
    end
    egcells =gcellset_Q4(struct('id',1:ngc,'conn',qconn));
    efens =fenodeset(struct('id',1:size(xyz,1),'xyz',xyz));
end
