% Merge several meshes together either by simple concatenation of nodes or by 
% gluing together nodes within tolerance. 
%
% function [fens,gcellsa] = merge_n_meshes(fensa, gcellsa, tolerance)
%
% Inputs:
% fensa= Cell array of node sets, 
% gcellsa= Cell array of geometric cell sets,  
% tolerance= Geometric tolerance, maybe supplied as zero (>=0). 
% 
% The meshes, fens1, gcells1, and fens2, gcells2, are glued together by
% merging the nodes that fall within a box of size "tolerance". If tolerance 
% is set to zero, no merging of nodes is performed; the nodes from the meshes are 
% simply concatenated together.
% 
% The merged node set, fens, and the two arrays of geometry cells with
% renumbered  connectivities are returned.
% 
% Outputs:
% fens= merged node set, 
% gcellsa= Cell array of geometric cell sets updated to use the merged node set.
function [fens,gcellsa] = merge_n_meshes(fensa, gcellsa, tolerance)
    fens=fensa{1};
    for j=2:length(gcellsa)
        [fens,fensa{j},fens2] = fuse_nodes(fensa{j}, fens, tolerance)
        gcellsa{j}= update_connectivity(gcellsa{j},get(fensa{j},'id'));
    end
end
