% Merge by gluing together nodes located within tolerance of each other.
%
% function [fens,gcells] = merge_nodes(fens, gcells, tolerance)
%
% The two meshes, fens1, gcells1, and fens2, gcells2, are glued together by
% merging the nodes that fall within a box of size "tolerance".
% The merged node array, fens, and the two arrays of geometry cells with
% renumbered  connectivities are returned.
% 
% For efficiency reasons, we should make sure that
% length(fens1)<length(fens2).
% 
function [fens,gcells] = merge_nodes(fens, gcells, tolerance)
    xyz1 = get(fens,'xyz');
    dim =size(xyz1,2); 
    id1 = get(fens,'id');
    c1=ones(size(xyz1,1),1);
    % Mark nodes from the first set that are duplicated 
    for i=1:count(fens)
        XYZ =xyz1(i,:);
        xyzd=abs(xyz1-c1*XYZ);
        j=find(sum(xyzd')'<tolerance);
        if (~isempty(j))   && (j(1) ~= i)
            id1(i) =-j(1);
        end
    end
    % Generate  merged arrays of the nodes
    xyzm = zeros(count(fens),dim);
    idm = zeros(count(fens),1);
    mid=1;
    for i=1:count(fens) % and then we pick only non-duplicated fens1
        if id1(i)>0
            id1(i)=mid;
            idm(mid)=mid;
            xyzm(mid,:)=xyz1(i,:);
            mid=mid+1;
        else 
            id1(i)=id1(-id1(i));
        end
    end
    nfens =mid-1;
    xyzm =xyzm(1:nfens,:);
    idm =idm(1:nfens,:);
    % Renumber the cells
    conns=get(gcells,'conn');
    for i=1:count(gcells)
        conn=conns(i,:);
        conns(i,:)=id1(conn);
    end
    gcells=set(gcells,'conn',conns);
    clear fens
    fens=fenodeset(struct('id',idm(1:nfens),'xyz',xyzm(1:nfens,:)));
end
